"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseRelativeDateFormat = exports.INVALID_RELATIVE_DATE_FORMAT_ERROR = exports.ACCEPTED_RELATIVE_DATE_VALUE = exports.isRelativeDateScalar = exports.RelativeDatePrefix = exports.RelativeDateResolution = void 0;
var RelativeDateResolution;
(function (RelativeDateResolution) {
    RelativeDateResolution["minutes"] = "minutes";
    RelativeDateResolution["hours"] = "hours";
    RelativeDateResolution["days"] = "days";
    RelativeDateResolution["weeks"] = "weeks";
    RelativeDateResolution["months"] = "months";
    RelativeDateResolution["years"] = "years";
})(RelativeDateResolution = exports.RelativeDateResolution || (exports.RelativeDateResolution = {}));
exports.RelativeDatePrefix = 'last';
const isRelativeDateScalar = (value) => value.startsWith(exports.RelativeDatePrefix);
exports.isRelativeDateScalar = isRelativeDateScalar;
exports.ACCEPTED_RELATIVE_DATE_VALUE = 'an Epoch time (number), ISO Date string or a relative date value in the form of "last-[N]-[minutes | hours | days | weeks | months | years]"';
const INVALID_RELATIVE_DATE_FORMAT_ERROR = () => {
    throw new Error(`Invalid relative date value! Can only accept ${exports.ACCEPTED_RELATIVE_DATE_VALUE}`);
};
exports.INVALID_RELATIVE_DATE_FORMAT_ERROR = INVALID_RELATIVE_DATE_FORMAT_ERROR;
const parseRelativeDateFormat = (value) => {
    let span;
    const [prefix, spanStr, resolution] = value.split('-');
    if (prefix !== exports.RelativeDatePrefix)
        (0, exports.INVALID_RELATIVE_DATE_FORMAT_ERROR)();
    try {
        span = parseInt(spanStr, 10);
    }
    catch (e) {
        (0, exports.INVALID_RELATIVE_DATE_FORMAT_ERROR)();
    }
    if (!RelativeDateResolution[resolution])
        (0, exports.INVALID_RELATIVE_DATE_FORMAT_ERROR)();
    return {
        resolution,
        span,
    };
};
exports.parseRelativeDateFormat = parseRelativeDateFormat;
//# sourceMappingURL=relative-date-scalar.helpers.js.map